from zope.interface import implements
from zope.app.component.hooks import setSite
from zope.component import queryUtility, provideUtility
from z3c.sampledata.interfaces import ISampleDataPlugin

from ice.template import ITemplates, Templates

class MailTemplatesPlugin(object):
    implements(ISampleDataPlugin)
    name = 'templates.mail'
    dependencies = []
    schema = None

    def generate(self, context, param={}, dataSource=None, seed=None):
        setSite(context)
        templates = queryUtility(ITemplates, 'templates.mail')
        if templates is None:
            templates = Templates()
            sm = context.getSiteManager()
            sm['mail-templates'] = templates
            sm.registerUtility(templates, ITemplates, 'templates.mail')
        return templates

class SkinTemplatesPlugin(object):
    implements(ISampleDataPlugin)
    name = 'templates.skin'
    dependencies = []
    schema = None

    def generate(self, context, param={}, dataSource=None, seed=None):
        setSite(context)
        templates = queryUtility(ITemplates, 'templates.skin')
        if templates is None:
            templates = Templates()
            sm = context.getSiteManager()
            sm['skin-templates'] = templates
            sm.registerUtility(templates, ITemplates, 'templates.skin')
        return templates



### -*- coding: utf-8 -*- ####################################################
#
#  Copyright (C) 2009 Ilshad Habibullin <astoon.net at gmail.com>
#
#  This library is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#  for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  Project homepage: <http://launchpad.net/ice.template>
#
##############################################################################

from zope import interface, schema
from ice.template import _

class ITemplates(interface.Interface):
    """Manage persistent Cheetah templates"""

    def getTemplate(name):
        """Return template"""

    def setTemplate(name, text):
        """Store template"""

    def compileTemplate(name, data):
        """Return compiled template. 'data' is dictionary of the template
        variables and 'name' is template name."""

    def getVariables(name):
        """Return variables names"""

    def resetTemplate(name):
        """Reset template from source"""

    def getSource(name):
        """Get source text"""

    def getAllTemplates():
        """Return all templates registered for this storage"""

class ITemplate(interface.Interface):
    """Cheetah template"""

    title = schema.TextLine(
        title=u"Title")

    storage = schema.TextLine(
        title=u"Local name of the storage")

    variables = schema.List(
        title=u"Variables names",
        value_type=schema.TextLine())

    source = schema.TextLine(
        title=u"Source file path")



### -*- coding: utf-8 -*- ####################################################
#
#  Copyright (C) 2009 Ilshad Habibullin <astoon.net at gmail.com>
#
#  This library is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#  for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  Project homepage: <http://launchpad.net/ice.template>
#
##############################################################################

from zope.location import LocationProxy
from zope.publisher.interfaces import browser, NotFound
from zope.component import queryMultiAdapter, getSiteManager
from zope.component.interfaces import ComponentLookupError, IDefaultViewName
from zope.interface import implements, providedBy

class TemplatesPublisher(object):
    implements(browser.IBrowserPublisher)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def publishTraverse(self, request, name):
        for n, adapter in self.context.getAllTemplates():
            if n == name:
                return LocationProxy(
                    adapter, container=self.context, name=name)

        view = queryMultiAdapter((self.context, request), name=name)
        if view is not None:
            return view
        raise NotFound(self.context, name, request)

    def browserDefault(self, request):
        name = getSiteManager(self.context).adapters.lookup(
            map(providedBy, (object, request)), IDefaultViewName)
        if name is not None:
            return self.context, (name,)
        raise ComponentLookupError(
            "Couldn't find default view name", self.context, request)



import unittest
from zope.app.testing import functional

functional.defineLayer('TestLayer', 'ftesting.zcml')

def test_suite():
    suite = unittest.TestSuite()

    suites = (functional.FunctionalDocFileSuite('ftests.txt'),)

    for s in suites:
        s.layer=TestLayer
        suite.addTest(s)
    return suite



### -*- coding: utf-8 -*- ####################################################
#
#  Copyright (C) 2009 Ilshad Habibullin <astoon.net at gmail.com>
#
#  This library is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#  for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  Project homepage: <http://launchpad.net/ice.template>
#
##############################################################################

from Cheetah.Template import Template
from BTrees.OOBTree import OOBTree
from persistent import Persistent
from zope.component import getAdapter, getAdapters, getUtilitiesFor
from zope.interface import implements
from zope.location import ILocation
from interfaces import ITemplates, ITemplate

def getUtilityName(context):
    for name, utility in getUtilitiesFor(ITemplates, context):
        if utility == context:
            return name

class Templates(Persistent):
    implements(ITemplates, ILocation)

    __parent__ = __name__ = None

    def __init__(self):
        self._templates = OOBTree()
        super(Templates, self).__init__()

    def getTemplate(self, name):
        return self._templates.get(name) or self.resetTemplate(name)

    def setTemplate(self, name, text):
        self._templates[name] = text

    def compileTemplate(self, name, data={}):
        return str(Template(self.getTemplate(name), searchList=[data]))

    def getVariables(self, name):
        return getAdapter(self, ITemplate, name=name).variables

    def resetTemplate(self, name):
        result = self._templates[name] = self.getSource(name)
        return result

    def getSource(self, name):
        tmpl = getAdapter(self, ITemplate, name=name)
        if tmpl.storage == getUtilityName(self):
            return tmpl.source
        raise KeyError

    def getAllTemplates(self):
        uname = getUtilityName(self)
        for name, adapter in getAdapters([self], ITemplate):
            if adapter.storage == uname:
                yield name, adapter



### -*- coding: utf-8 -*- ####################################################
#
#  Copyright (C) 2009 Ilshad Habibullin <astoon.net at gmail.com>
#
#  This library is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#  for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  Project homepage: <http://launchpad.net/ice.template>
#
##############################################################################

from z3c.table import table, column
from zope.traversing.browser.absoluteurl import absoluteURL
from ice.template import _

class Pagelet(table.Table):
    
    @property
    def values(self):
        return self.context.getAllTemplates()

class BrowserViewLinkColumn(column.Column):

    header = _(u'Name')

    def renderCell(self, item):
        return u'<a href="%s">%s</a>' % (
            absoluteURL(self.context, self.request) + u'/' + item[0],
            getattr(item[1], 'title', None) or item[0])



### -*- coding: utf-8 -*- ####################################################
#
#  Copyright (C) 2009 Ilshad Habibullin <astoon.net at gmail.com>
#
#  This library is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#  for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  Project homepage: <http://launchpad.net/ice.template>
#
##############################################################################

from zope.component import getUtility
from interfaces import ITemplates

class PersistentTemplate(object):
    
    def __init__(self, storage_name, template_name):
        self.__storage_name = storage_name
        self.__template_name = template_name

    def __get__(self, inst, klass):
         templates = getUtility(ITemplates, self.__storage_name)

         def compileTemplate(data={}):
             return templates.compileTemplate(self.__template_name, data)

         return compileTemplate



### -*- coding: utf-8 -*- ####################################################
#
#  Copyright (C) 2009 Ilshad Habibullin <astoon.net at gmail.com>
#
#  This library is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#  for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  Project homepage: <http://launchpad.net/ice.template>
#
##############################################################################

import os.path
from zope.schema import TextLine
from zope.interface import Interface, classImplements
from zope.component.zcml import adapter
from zope.configuration.fields import Tokens, Path
from zope.configuration.exceptions import ConfigurationError
from interfaces import ITemplate, ITemplates

class ITemplateDirective(Interface):
    """Persistent Cheetah template directive."""

    name = TextLine(
        title=u"Unique name")

    title = TextLine(
        title=u"Title")

    storage = TextLine(
        title=u"Local name of the storage")

    variables = Tokens(
        title=u"Variables names",
        value_type=TextLine(),
        default=[],
        required=False)

    source = Path(
        title=u"Source template file, using Cheetah syntax.",
        default=None,
        required=False)

def read_src(path):
    f = open(path)
    src = f.read()
    f.close()
    return src

def templateDirective(_context, name, title, storage, variables=[], source=None):

    path = os.path.abspath(str(_context.path(source)))
    if not os.path.isfile(path):
        raise ConfigurationError("No such file", path)

    def __init__(self, context):
        self.context = context

    cdict = {'__init__':__init__, 'title':title, 'storage':storage,
             'variables':variables, 'source':read_src(path)}

    klass = type('Template', (), cdict)

    classImplements(klass, ITemplate)

    adapter(_context, (klass,), provides=ITemplate, for_=(ITemplates,), name=name)



### -*- coding: utf-8 -*- ####################################################
#
#  Copyright (C) 2009 Ilshad Habibullin <astoon.net at gmail.com>
#
#  This library is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#  for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  Project homepage: <http://launchpad.net/ice.template>
#
##############################################################################

from Cheetah.Template import Template
from z3c.formui import form
from z3c.form import field, button
from zope.schema import Text
from ice.template import _

TFIELD_NAME = 'body'

class Pagelet(form.EditForm):
    form.extends(form.EditForm)
    
    fields = field.Fields(
        field.Field(Text(required=False), name=TFIELD_NAME))

    ignoreContext = True

    templates = property(lambda self: self.context.__parent__)
    tname = property(lambda self: self.context.__name__)
    preview = None

    def updateWidgets(self):
        super(Pagelet, self).updateWidgets()
        widget = self.widgets[TFIELD_NAME]
        if not self.request.get(widget.name):
            widget.value = self.templates.getTemplate(self.tname)

    def checkVariables(self):
        varnames = self.templates.getVariables(self.tname)
        variables = dict((v, u'***') for v in varnames)
        text = self.extractData()[0].get(TFIELD_NAME)
        try:
            return unicode(Template(text, variables))
        except:
            return None

    def applyChanges(self, data):
        if self.checkVariables() is not None:
            return super(Pagelet, self).applyChanges(data)
        self.noChangesMessage = _(u'There are some errors in the template')
        return None

    @button.buttonAndHandler(_(u'Reset from source'))
    def handleReset(self, action):
        self.widgets[TFIELD_NAME].value = self.templates.getSource(self.tname)

    @button.buttonAndHandler(_(u'Refresh'))
    def handleRefresh(self, action):
        self.widgets[TFIELD_NAME].value = self.templates.getTemplate(self.tname)

    @button.buttonAndHandler(_(u'Preview and Test'))
    def handlePreview(self, action):
        self.preview = self.checkVariables()
        self.status = self.preview is not None and _(u'Ok') or _(
            u'There are some errors in the template')

class TemplateTextField(object):

    query = lambda self, *args: self.get()
    canAccess = lambda self: True
    canWrite = lambda self: True

    def __init__(self, context, field):
        self.templates = context.__parent__
        self.name = context.__name__
        self.field = field

    def get(self):
        return self.templates.getTemplate(self.name)

    def set(self, value):
        self.templates.setTemplate(self.name, value)



import unittest, doctest
from zope.app.testing import setup
from zope.testing.doctestunit import DocFileSuite

def setUp(test):
    setup.placefulSetUp()

def tearDown(test):
    setup.placefulTearDown()

def test_suite():
    return unittest.TestSuite(
        (DocFileSuite('README.txt', package='ice.template',
                      setUp=setUp, tearDown=tearDown,
                      optionflags=doctest.NORMALIZE_WHITESPACE
                      | doctest.ELLIPSIS,),))



import sys, os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version='0.1.0'

setup(name='ice.template',
      version='0.1.0',
      author='Ilshad Habibullin',
      author_email = 'astoon.net at gmail.com',
      url='http://launchpad.net/ice.template',
      description = 'Persistent cheetah templates',
      long_description = (read('src/ice/template/README.txt')
                          + '\n\n' + read('CHANGES.txt')),
      license = 'GPL v.3',
      classifiers = [
              'Development Status :: 4 - Beta',
              'Environment :: Web Environment',
              'Intended Audience :: Developers',
              'License :: OSI Approved :: GNU General Public License (GPL)',
              'Programming Language :: Python',
              'Natural Language :: English',
              'Operating System :: OS Independent',
              'Topic :: Internet :: WWW/HTTP :: Site Management',
              'Framework :: Zope3'],
      packages=find_packages('src'),
      namespace_packages=['ice',],
      package_dir={'':'src'},

      extras_require = dict(
              test=['zope.app.zcmlfiles',
                    'zope.app.testing',
                    'zope.testing',
                    'zope.testbrowser',
                    'zope.securitypolicy',
                    'z3c.sampledata']),
      
      install_requires=['setuptools',
                        'Cheetah',
                        'ZODB3',
                        'zope.dublincore',
                        'z3c.formui',
                        'z3c.layer',
                        'z3c.pagelet',
                        'z3c.table'],
      
      dependency_links = ['http://download.zope.org/distribution'], 
      include_package_data=True,
      zip_safe=False)



from z3c.form.interfaces import IFormLayer
from z3c.formui.interfaces import IDivFormLayer
from z3c.layer.pagelet import IPageletBrowserLayer
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

class ILayer(IFormLayer, IPageletBrowserLayer):
    """Layer"""

class ISkin(IDivFormLayer, ILayer, IDefaultBrowserLayer):
    """Skin Layer"""
